unit Unit1;

{
  Program to poll the GDS-1062 scope. It can get the data from the scope.
  Send the commands to the serial port
  The scope sends the data and stops.

  loads commands from: 'scopeCmdLst.txt'
  saves memo to 'scope.csv'

  draft 0.1
  This allows data to be recovered and saved.
  You have to single step through the commands

}


interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, CommDrv, StdCtrls,
  Registry, Mask;

type
  TForm1 = class(TForm)
    Label2: TLabel;
    CommPortDriver1: TCommPortDriver;
    Timer1: TTimer;
    Memo1: TMemo;
    Label5: TLabel;
    Edit1: TEdit;
    Label1: TLabel;
    Button1: TButton;
    Label3: TLabel;
    Label4: TLabel;
    Button2: TButton;
    Shape1: TShape;
    ComboBox1: TComboBox;
    save: TButton;
    Label7: TLabel;
    Button3: TButton;
    Label6: TLabel;

    procedure CommPortDriver1ReceiveData(Sender: TObject; DataPtr: Pointer;
      DataSize: Integer);
    procedure FormCreate(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Edit1Change(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure ComboBox1DropDown(Sender: TObject);
    procedure saveClick(Sender: TObject);
    procedure ComboBox1Exit(Sender: TObject);
    procedure ComboBox1DblClick(Sender: TObject);
    procedure Button3Click(Sender: TObject);
  //  procedure PaintBox1Paint(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

Const
  MAX_BUFF = 10000;

{ define data buffer to put scope data into }
type
  Tmem = ( h, dataSizeDigits, timeInterval , dataWords );

  TacqmemBytesT =   array[0..MAX_BUFF] of byte ;
  TacqmemCharsT =   array[0..MAX_BUFF] of Char ;

  TacqmemT = packed record
      h : char; { # }
      dataSizeDigits : array[0..4] of char; { 48008 }
      {timeIntervalBytes : array[0..3] of char; {single;}   {34 56 bf 94  => 94 bf 56 34 }
      // before use byted need to be reversed. {34 56 bf 94  => 94 bf 56 34 }

      timeInterval : single; {single;}   {34 56 bf 94  => 94 bf 56 34 }
      ChannelIndicator : byte;
      reserved  : array[0..2] of byte ;
      dataWords : array[0..7999] of SmallInt ;
  end;

  TacqmemUnionT = packed record
    case t1 : integer of
      1  : ( v1: TacqmemBytesT ) ;
      2  : ( v2: TacqmemT );
      3  : ( v3: TacqmemCharsT );
    end;

var
  lastCmd : string ;
  acqMemData: TacqmemT;
  acqMemDataFill:integer;

  acqmemBytesOffset : integer;
  acqmem            : TacqmemUnionT ;

var
  Form1: TForm1;
  buffer:string;
  wantedLineCount : Integer;
  IniFile : TRegIniFile;
  timerCountDown : Integer;

implementation

{$R *.DFM}

{
  The command is sent.
  Gather until complete
}


procedure TForm1.CommPortDriver1ReceiveData(Sender: TObject;
  DataPtr: Pointer; DataSize: Integer);

var
  comRxDataA: array[0..3000] of Char;
  finished : boolean ;
  foundCRLFpos :integer;
  line , rest : string;

begin
    Shape1.Shape := stCircle;
    Shape1.brush.color := clRed;
    timerCountDown := 3;
    { buffer until end or time out }
    if ( acqmemBytesOffset + DataSize ) < sizeof( TacqmemBytesT ) then
    begin
      move(  DataPtr^, acqmem.v1[acqmemBytesOffset], DataSize );
      acqmemBytesOffset :=  acqmemBytesOffset + DataSize ;
    end;

    Label3.Caption := IntToStr( acqmemBytesOffset ) +
                      ' ' +acqmem.v2.dataSizeDigits ;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  timerCountDown := 0;
  Button1Click(Sender);

  if   fileExists('scopeCmdLst.txt') then
    ComboBox1.Items.LoadFromFile('scopeCmdLst.txt')
  else
    ComboBox1.Items.SaveToFile('scopeCmdLst.txt');

  ComboBox1.ItemIndex := 1;
  ComboBox1.Text := ComboBox1.Items[ComboBox1.ItemIndex];

  {
*idn?
*lrn?
:acq1:mem
:acq2:mem
:system:error?
:SYSTem:VERSion?
:MEAS:SOUR 1;:MEAS:VAMP?
:MEAS:SOUR 1;:MEAS:PWID?

:ref1:display 1
:ref1:loc 0
:ref1:loc 50
:ref1:loc -50

:autoset
  }



end;

procedure TForm1.Button1Click(Sender: TObject);
var
  temp : string;
  opStr: string;
  comCounter : integer;
  newPortNum : integer;

begin
{ create }
  IniFile  := TRegIniFile.Create('comtestDHR.ini');
  Edit1.Text := IniFile.ReadString('ComPort','lastComPort','1');

  Memo1.clear;
  { try and connect to each port in turn }
  opStr := 'ports: ';
  CommPortDriver1.ComPort := pnCOM1;

  for comCounter := 0 to 20 do
  begin
    if CommPortDriver1.connected then
      CommPortDriver1.disconnect;

    CommPortDriver1.ComPort :=  TComPortNumber( comCounter );
    if CommPortDriver1.connected then
      CommPortDriver1.disconnect;

    if not CommPortDriver1.connected then
      CommPortDriver1.connect;

    if CommPortDriver1.connected then
        CommPortDriver1.SendString( ''+chr(10) );

    if CommPortDriver1.connected then
      opStr := opStr + IntToStr( comCounter + 1 ) + ','
    else
      opStr := opStr + '_,';

  end;
  Label3.caption := opStr;

  if not CommPortDriver1.connected then
    CommPortDriver1.disconnect;


  // validate port number
  newPortNum := StrToIntDef( Edit1.Text, -1 ) ;
  if ( newPortNum < 1 ) OR (newPortNum > 256) then
  begin
     // force port number to valid range
     Edit1.Text := '1';
     newPortNum := 1  ;
  end;

  IniFile.WriteString('ComPort','lastComPort',IntToStr(newPortNum) );
  CommPortDriver1.ComPort := TComPortNumber( newPortNum - 1 );

  // CommPortDriver1.ComPort := pnCOM1;

  if not CommPortDriver1.connected then
    CommPortDriver1.connect;

  if CommPortDriver1.connected then
  begin
    label2.caption := 'connected' ;
    buffer := '';
    Memo1.clear;

    Memo1.Lines.Add( '============================' );
    Memo1.Lines.Add( 'Polled At:'+ FormatDateTime(' dd/mm/yyyy hh:mm ', Now) );
    Memo1.Lines.Add( '============================' );
    wantedLineCount := 0;

//    lastCmd := '*idn?' ;
//    CommPortDriver1.SendString( lastCmd + chr(10) );

//  lastCmd := '*lrn?';
//  CommPortDriver1.SendString( lastCmd + chr(10) );


  end
  else
  begin
    label2.caption := 'failed to connect' ;
  end;

end;

procedure TForm1.Edit1Change(Sender: TObject);
var
  newPortNum : Integer;
begin
  // must be blank or 1..256
  if( Edit1.Text <> '' ) then
  begin
    newPortNum := StrToIntDef( Edit1.Text, -1 ) ;
    if ( newPortNum < 1 ) OR (newPortNum > 256) then
      Edit1.Text := '1';
  end;
  IniFile.WriteString('ComPort','lastComPort',Edit1.Text );
  IniFile.WriteString('ComPort','','doug' );

end;

procedure TForm1.Button2Click(Sender: TObject);
var
  temp : string;
  opStr: string;
begin
  acqmemBytesOffset := 0 ;
  lastCmd := ComboBox1.Text ;
  CommPortDriver1.SendString( lastCmd + chr(10) );
  ComboBox1.ItemIndex := ComboBox1.ItemIndex + 1 ;
  ComboBox1.Text := ComboBox1.Items[ComboBox1.ItemIndex];
end;

procedure TForm1.Timer1Timer(Sender: TObject);
var
  tempStr : string;
  opStr,line   : string;
  temp    : byte;
  temp1   : integer;
  dp      : integer;
  shown,finished   : boolean;
  foundCRLFpos : integer;
begin
  if ( timerCountDown = 1 ) then
  begin
    Memo1.Lines.Add( '===================================================' );
    Memo1.Lines.Add( 'Polled At:'+ FormatDateTime(' dd/mm/yyyy hh:mm ', Now) );
    Memo1.Lines.Add( '============================' );

    timerCountDown := timerCountDown - 1;

    Shape1.Shape := stSquare;
    Shape1.Brush.Color := clGreen;

    { Ensure Buffer has a NULL on the end }

    acqmem.v1[acqmemBytesOffset-1] := 0 ;
    tempStr := acqmem.v3;

    if char(acqmem.v3[0]) = char(23) {#} then
    begin
      acqmem.v1[6] := 0;
    end;

    Label4.Caption := IntToStr( acqmemBytesOffset ) +
                        ' ' + acqmem.v3 ;

    shown := false;
    if ( lastCmd = ':acq1:mem') or (lastCmd = ':acq2:mem') then
    begin

      temp := acqmem.v1[6];
      acqmem.v1[6] := acqmem.v1[9];
      acqmem.v1[9] := temp;

      temp := acqmem.v1[7];
      acqmem.v1[7] := acqmem.v1[8];
      acqmem.v1[8] := temp;

      Str( acqmem.v2.timeInterval , opStr );

      Label3.caption := opStr;

      shown := true;
      Memo1.Lines.Add( lastCmd );
      //Memo1.Lines.Add( acqmem.v3  );
      Memo1.Lines.Add( '===================================================' );

      Memo1.Lines.Add( IntToStr(acqmem.v1[11])+ IntToStr(acqmem.v1[12])+IntToStr(acqmem.v1[13])+IntToStr(acqmem.v1[14]));
      Memo1.Lines.Add( char(acqmem.v1[11])+ char(acqmem.v1[12])+char(acqmem.v1[13])+char(acqmem.v1[14]));

      Memo1.Lines.Add( acqmem.v2.h + ' ' + acqmem.v2.dataSizeDigits +
                   ' dt: ' + opStr +
                   ' ch: ' + IntToStr( acqmem.v2.ChannelIndicator ) +
                   ' ' + IntToStr( acqmemBytesOffset)  +
                   ' ' + IntToStr( acqmem.v2.dataWords[0] )  +
                   ' ' + IntToStr( acqmem.v2.dataWords[3999] )  +

                   ''
                   );
      dp := 0;

      Memo1.Lines.BeginUpdate;
      for temp1 := 0 to 3999 do
      begin
        Memo1.Lines.Add(  ' ' + IntToStr( temp1 ) + ',' + IntToStr( swap(acqmem.v2.dataWords[ dp ]  )) + '' );
        dp := dp + 1;
      end;
      Memo1.Lines.EndUpdate;
      Memo1.Lines.Add( '' );
      Memo1.Show;
    end;
    if lastCmd = '*lrn?' then
    begin
      shown := true;
      finished := false ;

      buffer := acqmem.v3;

      while ( not finished ) do
      begin
        foundCRLFpos := pos( ';' , buffer );
        if ( foundCRLFpos > 0 ) then
        begin
          { split buffer }
          line := copy( buffer , 1,foundCRLFpos );
          // label6.caption :=  line;
          delete( buffer , 1,foundCRLFpos );
          Memo1.Lines.Add( trim( line ) );
        end
        else
          finished := true;
      end;
      Memo1.Lines.Add( trim( buffer ) );
    end;
    if ( not shown ) then
    begin
      Memo1.Lines.Add( lastCmd );
      Memo1.Lines.Add( acqmem.v3  );
    end;
  end;

  if timerCountDown > 0 then
  begin
    timerCountDown := timerCountDown - 1;

    label4.caption := 'line: ' +
        IntToStr( timerCountDown ) + ':' ;

  end

end;


procedure TForm1.ComboBox1DropDown(Sender: TObject);
begin
  ComboBox1.Text := ComboBox1.Items[ComboBox1.ItemIndex];
end;

procedure TForm1.saveClick(Sender: TObject);
begin
  Memo1.Lines.SaveToFile('scope.csv');
end;

procedure TForm1.ComboBox1Exit(Sender: TObject);
var
  temp : string;
  opStr: string;
begin
  acqmemBytesOffset := 0 ;
  lastCmd := ComboBox1.Items[ComboBox1.ItemIndex] ;
  lastCmd := ComboBox1.Text ;
  CommPortDriver1.SendString( lastCmd + chr(10) );
end;

procedure TForm1.ComboBox1DblClick(Sender: TObject);
var
  temp : string;
  opStr: string;
begin
  acqmemBytesOffset := 0 ;
  lastCmd := ComboBox1.Items[ComboBox1.ItemIndex] ;
  lastCmd := ComboBox1.Text ;
  CommPortDriver1.SendString( lastCmd + chr(10) );
end;

procedure TForm1.Button3Click(Sender: TObject);
begin
  if Form1.Height < 400 then
  begin
    Form1.Height := 843;
    Memo1.Height := 593;
  end
  else
  begin
    Form1.Height := 392;
    Memo1.Height := 153;
  end
end;

{
procedure TForm1.PaintBox1Paint(Sender: TObject);
begin
   PaintBox1.Canvas.LineTo(
     PaintBox1.width,
     PaintBox1.height
     )
end;
}

end.
